import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import custom_exception, dict_output, _arg_split

@click.command('show_dataset')
@click.argument("history_id", type=str)
@click.argument("dataset_id", type=str)


@pass_context
@custom_exception
@dict_output
def cli(ctx, history_id, dataset_id):
    """Get details about a given history dataset.

Output:

    
    """
    return ctx.gi.histories.show_dataset(history_id, dataset_id)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import custom_exception, dict_output, _arg_split

@click.command('import_workflow_dict')
@click.argument("workflow_dict", type=str)


@pass_context
@custom_exception
@dict_output
def cli(ctx, workflow_dict):
    """Imports a new workflow given a dictionary representing a previously exported workflow.

Output:

    
    """
    return ctx.gi.workflows.import_workflow_dict(json_loads(workflow_dict))



import click

from parsec.cli import pass_context
from parsec.decorators import bioblend_exception, dict_output


@click.command('tool_data_get_data_tables')
@pass_context
@bioblend_exception
@dict_output
def cli(ctx):
    """Displays a collection (list) of data tables.
    """
    return ctx.gi.tool_data.get_data_tables()



import click
from parsec.commands.folders.create_folder import cli as func0
from parsec.commands.folders.delete_folder import cli as func1
from parsec.commands.folders.get_permissions import cli as func2
from parsec.commands.folders.set_permissions import cli as func3
from parsec.commands.folders.show_folder import cli as func4
from parsec.commands.folders.update_folder import cli as func5

@click.group()
def cli():
    pass

cli.add_command(func0)
cli.add_command(func1)
cli.add_command(func2)
cli.add_command(func3)
cli.add_command(func4)
cli.add_command(func5)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import custom_exception, dict_output, _arg_split

@click.command('show_folder')
@click.argument("folder_id", type=str)


@pass_context
@custom_exception
@dict_output
def cli(ctx, folder_id):
    """Display information about a folder.

Output:

     dictionary including details of the folder
        
    """
    return ctx.gi.folders.show_folder(folder_id)



import click

from parsec.cli import pass_context
from parsec.decorators import bioblend_exception, dict_output


@click.command('tool_data_show_data_table')
@click.argument("data_table_id", type=str)
@pass_context
@bioblend_exception
@dict_output
def cli(ctx, data_table_id):
    """Display information on a single data_table
    """
    return ctx.gi.tool_data.show_data_table(data_table_id)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import bioblend_exception, dict_output

@click.command('install_repository_revision')
@click.argument("tool_shed_url", type=str)
@click.argument("name", type=str)
@click.argument("owner", type=str)
@click.argument("changeset_revision", type=str)

@click.option(
    "--install_tool_dependencies",
    help="Whether or not to automatically handle tool dependencies (see https://galaxyproject.org/toolshed/tool-dependency-recipes/ for more details)",
    is_flag=True
)
@click.option(
    "--install_repository_dependencies",
    help="Whether or not to automatically handle repository dependencies (see https://galaxyproject.org/toolshed/defining-repository-dependencies/ for more details)",
    is_flag=True
)
@click.option(
    "--install_resolver_dependencies",
    help="Whether or not to automatically install resolver dependencies (e.g. conda). This parameter is silently ignored in Galaxy ``release_16.04`` and earlier.",
    is_flag=True
)
@click.option(
    "--tool_panel_section_id",
    help="The ID of the Galaxy tool panel section where the tool should be insterted under. Note that you should specify either this parameter or the ``new_tool_panel_section_label``. If both are specified, this one will take precedence.",
    type=str
)
@click.option(
    "--new_tool_panel_section_label",
    help="The name of a Galaxy tool panel section that should be created and the repository installed into.",
    type=str
)

@pass_context
@bioblend_exception
@dict_output
def cli(ctx, tool_shed_url, name, owner, changeset_revision, install_tool_dependencies=False, install_repository_dependencies=False, install_resolver_dependencies=False, tool_panel_section_id="", new_tool_panel_section_label=""):
    """Install a specified repository revision from a specified Tool Shed into this Galaxy instance. This example demonstrates installation of a repository that contains valid tools, loading them into a section of the Galaxy tool panel or creating a new tool panel section. You can choose if tool dependencies or repository dependencies should be installed through the Tool Shed, (use ``install_tool_dependencies`` or ``install_repository_dependencies``) or through a resolver that supports installing dependencies (use ``install_resolver_dependencies``). Note that any combination of the three dependency resolving variables is valid.
    """
    return ctx.gi.toolshed.install_repository_revision(tool_shed_url, name, owner, changeset_revision, install_tool_dependencies=install_tool_dependencies, install_repository_dependencies=install_repository_dependencies, install_resolver_dependencies=install_resolver_dependencies, tool_panel_section_id=tool_panel_section_id, new_tool_panel_section_label=new_tool_panel_section_label)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import bioblend_exception, dict_output

@click.command('get_repositories')


@pass_context
@bioblend_exception
@dict_output
def cli(ctx):
    """Get the list of all installed Tool Shed repositories on this Galaxy instance.
    """
    return ctx.gi.toolshed.get_repositories()



import click

from parsec.cli import pass_context
from parsec.decorators import bioblend_exception, dict_output


@click.command('users_create_user')
@click.argument("user_email")
@pass_context
@bioblend_exception
@dict_output
def cli(ctx, user_email):
    """Deprecated method.
    """
    return ctx.gi.users.create_user(user_email)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import custom_exception, dict_output, _arg_split

@click.command('add_group_role')
@click.argument("group_id", type=str)
@click.argument("role_id", type=str)


@pass_context
@custom_exception
@dict_output
def cli(ctx, group_id, role_id):
    """Add a role to the given group.

Output:

     Added group role's info
        
    """
    return ctx.gi.groups.add_group_role(group_id, role_id)



import sys
import time
import click
import tempfile
import json
try:
    import StringIO as io
except:
    import io

from parsec.cli import pass_context
from parsec.decorators import bioblend_exception
from justbackoff import Backoff
from subprocess import CalledProcessError, check_output
from xunit_wrapper import xunit, xunit_suite, xunit_dump
from six import binary_type, string_types, text_type


def unicodify(value):
    # From Galaxy
    if value is None:
        return None
    try:
        if not isinstance(value, string_types) and not isinstance(value, binary_type):
            value = str(value)
        if not isinstance(value, text_type):
            value = text_type(value, 'utf-8', 'replace')
    except Exception:
        return "error converting to unicode"
    return value


@click.command('xunit_xargs', context_settings=dict(
    ignore_unknown_options=True,
    allow_extra_args=True
))
@click.argument("_")

@pass_context
@bioblend_exception
def cli(ctx, _):
    """xargs look-alike that wraps output calls as XUnit XML

    e.g.::

        parsec histories get_histories | \
            jq '.[].id' -r | \
            head -n 3 | \
            parsec utils xunit_xargs parsec histories get_status \| jq .percent_complete

    will fetch the first three histories mentioned, and then pass them to xargs
    to run ``parsec histories get_status [history_id] | jq .percent_complete``. This will
    in turn produce XUnit XML that can be used in Jenkins or similar systems::

        <?xml version="1.0" ?>
        <testsuites errors="0" failures="0" tests="3" time="1.5944418907165527">
                <testsuite errors="0" failures="0" name="Parsec XX" skipped="0" tests="3" time="1.5944418907165527">
                        <testcase classname="parsec.histories.get_status.769f01a3981796db_|.jq..percent_complete" name="parsec.histories.get_status.769f01a3981796db_" time="0.604831">
                                <system-out>100</system-out>
                        </testcase>
                        <testcase classname="parsec.histories.get_status.83fbc32772cb5fcf_|.jq..percent_complete" name="parsec.histories.get_status.83fbc32772cb5fcf_" time="0.483556">
                                <system-out>100</system-out>
                        </testcase>
                        <testcase classname="parsec.histories.get_status.90c9282cb8718062_|.jq..percent_complete" name="parsec.histories.get_status.90c9282cb8718062_" time="0.506056">
                                <system-out>97.82608695652173</system-out>
                        </testcase>
                </testsuite>
        </testsuites>
    """
    test_cases = []
    for line in sys.stdin:
        # pretend to be xargs

        piped_commands = sys.argv[sys.argv.index('xunit_xargs') + 1:]
        if '|' in piped_commands:
            pipe_idx = piped_commands.index('|')
            piped_commands[pipe_idx] = line.strip() + ' |'
            built_command = piped_commands
        else:
            built_command = piped_commands + [line.strip()]
        # TODO: detect spaces in args and warn that they should be quoted.
        # If they provide multiple strings, then pipe them together

        xunit_identifier = '.'.join([x.strip().replace(' ', '_') for x in piped_commands])
        xunit_identifier.replace(' ', '_')
        if '|' in xunit_identifier:
            xunit_name = xunit_identifier[0:xunit_identifier.index('|')]
        else:
            xunit_name = xunit_identifier

        stderr = tempfile.NamedTemporaryFile()
        output = ""
        with xunit(xunit_name, xunit_identifier) as test_case:
            ctx.vlog('Executing: %s', ' '.join(built_command))
            output = check_output(' '.join(built_command), shell=True, stderr=stderr)

        # Set stdout
        stderr.seek(0)
        test_case._tc.stdout = unicodify(output).strip()
        test_case._tc.stderr = unicodify(stderr.read()).strip()
        # Append to list
        test_cases.append(test_case)

    ts = xunit_suite('Parsec XX', test_cases)
    print(xunit_dump([ts]))



import sys
import time
import click
import json
from parsec.cli import pass_context
from parsec.decorators import bioblend_exception
from justbackoff import Backoff
from subprocess import CalledProcessError, check_output
from xunit_wrapper import xunit, xunit_suite, xunit_dump
from six import binary_type, string_types, text_type


@click.command('cmp')
@click.argument("method", type=str, required=True)
@click.argument("cmp_with", type=str, required=True)

@pass_context
@bioblend_exception
def cli(ctx, method, cmp_with):
    """comparison tool. Exits if the value read from stdin does not pass the comparison test with the specified value.

    method is the comparison method. One of these (lt, gt, eq, ne) which will
    trigger a numerical comparison (cast to floats), or one of teq, tneq which
    will trigger a string comparison.

    cmp_with is the value to compare against. E.g. '100' or 'test'

    e.g.::

        echo '5' | parsec utils cmp lt 10 # exit 0

        echo '5' | parsec utils cmp lt 1 # exit 1

    """
    data = sys.stdin.read()
    if method in ('lt', 'gt', 'ne', 'eq'):
        data = float(data.strip())
        cmp_with = float(cmp_with.strip())

        if method == 'lt':
            ctx.exit(0) if data < cmp_with else sys.stderr.write('%s >= %s' % (data, cmp_with)); ctx.exit(1)
        elif method == 'gt':
            ctx.exit(0) if data > cmp_with else sys.stderr.write('%s <= %s' % (data, cmp_with)); ctx.exit(1)
        elif method == 'ne':
            ctx.exit(0) if data != cmp_with else sys.stderr.write('%s == %s' % (data, cmp_with)); ctx.exit(1)
        elif method == 'eq':
            ctx.exit(0) if data == cmp_with else sys.stderr.write('%s != %s' % (data, cmp_with)); ctx.exit(1)

    elif method in ('teq', 'tneq'):
        if method == 'tneq':
            ctx.exit(0) if data != cmp_with else sys.stderr.write('%s == %s' % (data, cmp_with)); ctx.exit(1)
        elif method == 'teq':
            ctx.exit(0) if data == cmp_with else sys.stderr.write('%s != %s' % (data, cmp_with)); ctx.exit(1)
    ctx.exit(2)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import custom_exception, dict_output, _arg_split

@click.command('get_library_permissions')
@click.argument("library_id", type=str)


@pass_context
@custom_exception
@dict_output
def cli(ctx, library_id):
    """Get the permessions for a library.

Output:

     dictionary with all applicable permissions' values
        
    """
    return ctx.gi.libraries.get_library_permissions(library_id)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import bioblend_exception, dict_output

@click.command('show_repository')
@click.argument("toolShed_id", type=str)


@pass_context
@bioblend_exception
@dict_output
def cli(ctx, toolShed_id):
    """Get details of a given Tool Shed repository as it is installed on this Galaxy instance.
    """
    return ctx.gi.toolshed.show_repository(toolShed_id)



import click
from parsec.cli import pass_context, json_loads
from parsec.decorators import custom_exception, dict_output, _arg_split

@click.command('show_category')
@click.argument("category_id", type=str)


@pass_context
@custom_exception
@dict_output
def cli(ctx, category_id):
    """Get details of a given category.

Output:

     details of the given category

    """
    return ctx.ti.categories.show_category(category_id)



import time
import click
import json
from parsec.cli import pass_context
from parsec.decorators import bioblend_exception
from justbackoff import Backoff


@click.command('wait_on_invocation')
@click.argument("workflow_id", type=str, required=True)
@click.argument("invocation_id", type=str, required=True)
@click.option(
    "--exit_early",
    help="Exit immediately after checking status rather than sleeping indefinitely",
    is_flag=True
)
@click.option(
    "--backoff_min",
    help="Minimum time to sleep between checks, in seconds.",
    default=1,
    type=float,
)
@click.option(
    "--backoff_max",
    help="Maximum time to sleep between checks, in seconds",
    default=60,
    type=float,
)

@pass_context
@bioblend_exception
def cli(ctx, workflow_id, invocation_id, exit_early=False, backoff_min=1, backoff_max=60):
    """Given a workflow and invocation id, wait until that invocation is
    complete (or one or more steps have errored)

    This will exit with the following error codes:

    - 0: done successfully
    - 1: running (if --exit_early)
    - 2: failure
    - 3: unknown
    """
    backoff = Backoff(min_ms=backoff_min * 1000, max_ms=backoff_max * 1000, factor=2, jitter=True)

    prev_state = None
    while True:
        # Fetch the current state
        latest_state = ctx.gi.workflows.show_invocation(workflow_id, invocation_id)
        # Get step states
        states = [step['state'] for step in latest_state['steps'] if step['state'] is not None and step['state'] != 'deleted']
        # Get a str based state representation
        state_rep = '|'.join(map(str, states))

        if state_rep != prev_state:
            backoff.reset()
        prev_state = state_rep

        # If it's scheduled, then let's look at steps. Otherwise steps probably don't exist yet.
        if latest_state['state'] == 'scheduled':
            ctx.vlog("Checking workflow %s states: %s", workflow_id, state_rep)

            if exit_early:
                print(json.dumps({'state': 'running', 'job_states': states}))
                ctx.exit(1)

            # Conditions which must be true for all jobs before we can be done
            if all([state == 'ok' for state in states]):
                print(json.dumps({'state': 'done', 'job_states': states}))
                ctx.exit(0)

            # Conditions on which to exit immediately (i.e. due to a failure)
            if any([state in ('error', 'paused') for state in states]):
                print(json.dumps({'state': 'failure', 'job_states': states}))
                ctx.exit(2)
        else:
            ctx.vlog("Waiting for invocation to be scheduled")

            if exit_early:
                print(json.dumps({'state': 'unscheduled'}))
                ctx.exit(0)

        time.sleep(backoff.duration())
    ctx.exit(3)
